@@ -3,18 +3,18 @@
  * Copyright (C) 2007, Jamie McCracken <jamiemcc@gnome.org>
  * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
@@ -35,7 +35,6 @@
 #include <libtracker-extract/tracker-extract.h>
 
 #include "tracker-albumart.h"
-#include "tracker-dbus.h"
 
 /* We wait this long (seconds) for NULL state before freeing */
 #define TRACKER_EXTRACT_GUARD_TIMEOUT 3
@@ -144,8 +143,8 @@ static TrackerExtractData data[] = {
 	{ "audio/*", extract_gstreamer_audio },
 	{ "video/*", extract_gstreamer_video },
 	{ "image/*", extract_gstreamer_image },
-	/* Tell gstreamer to guess if mimetype guessing returns video also for audio files */
 	{ "image/svg+xml", extract_gstreamer_svg },
+	/* Tell gstreamer to guess if mimetype guessing returns video also for audio files */
 	{ "video/3gpp", extract_gstreamer_guess },
 	{ "video/mp4", extract_gstreamer_guess },
 	{ "video/x-ms-asf", extract_gstreamer_guess },
@@ -317,10 +316,10 @@ add_y_date_gst_tag (TrackerSparqlBuilder  *metadata,
 		g_date_free (date);
 	}
 
-	tracker_guarantee_date_from_filename_mtime (metadata,
-	                                            key,
-	                                            buf,
-	                                            uri);
+	tracker_guarantee_date_from_file_mtime (metadata,
+	                                        key,
+	                                        buf,
+	                                        uri);
 }
 
 static void
@@ -509,8 +508,6 @@ extract_metadata (MetadataExtractor      *extractor,
 	const gchar *temp;
 	gchar *make = NULL, *model = NULL, *manuf = NULL;
 	gchar *composer = NULL, *albumname = NULL, *genre = NULL;
-	gboolean ret;
-	gint count;
 	gboolean needs_audio = FALSE;
 
 	g_return_if_fail (extractor != NULL);
@@ -778,10 +775,10 @@ extract_metadata (MetadataExtractor      *extractor,
 
 		s = NULL;
 		gst_tag_list_get_string (extractor->tagcache, GST_TAG_TITLE, &s);
-		tracker_guarantee_title_from_filename (metadata,
-		                                       "nie:title",
-		                                       s,
-		                                       uri);
+		tracker_guarantee_title_from_file (metadata,
+		                                   "nie:title",
+		                                   s,
+		                                   uri);
 		g_free (s);
 
 		add_string_gst_tag (metadata, uri, "nie:copyright", extractor->tagcache, GST_TAG_COPYRIGHT);
@@ -847,9 +844,10 @@ extract_metadata (MetadataExtractor      *extractor,
 		}
 
 		if (needs_audio) {
+			guint count;
+
 			/* Audio */
-			ret = gst_tag_list_get_uint (extractor->tagcache, GST_TAG_TRACK_COUNT, &count);
-			if (ret) {
+			if (gst_tag_list_get_uint (extractor->tagcache, GST_TAG_TRACK_COUNT, &count)) {
 				*scount = g_strdup_printf ("%d", count);
 			}
 
@@ -885,6 +883,11 @@ extract_metadata (MetadataExtractor      *extractor,
 		g_free (album_disc_uri);
 
 		add_string_gst_tag (metadata, uri, "nfo:codec", extractor->tagcache, GST_TAG_AUDIO_CODEC);
+	} else if (extractor->mime == EXTRACT_MIME_GUESS) {
+		g_warning ("Cannot guess real stream type if no tags were read! "
+		           "Defaulting to Video.");
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nmm:Video");
 	} else {
 		if (extractor->mime == EXTRACT_MIME_AUDIO)
 			needs_audio = TRUE;
@@ -909,6 +912,11 @@ extract_metadata (MetadataExtractor      *extractor,
 		}
 	}
 
+	/* If content was encrypted, set it. */
+	if (extractor->is_content_encrypted) {
+		tracker_sparql_builder_predicate (metadata, "nfo:isContentEncrypted");
+		tracker_sparql_builder_object_boolean (metadata, TRUE);
+	}
 
 	if (use_tagreadbin) {
 		extract_stream_metadata_tagreadbin (extractor, uri, metadata);
@@ -1089,28 +1097,42 @@ poll_for_ready (MetadataExtractor *extractor,
 		}
 		case GST_MESSAGE_ERROR: {
 			GError *lerror = NULL;
-			gchar  *error_message;
+			gchar  *error_debug_message;
 
-			gst_message_parse_error (message, &lerror, &error_message);
+			gst_message_parse_error (message, &lerror, &error_debug_message);
 
-#if (GST_VERSION_MICRO >= 20)
+			/* The first GStreamer version with these encryption related errors
+			 * is 0.10.20 */
+#if (GST_CHECK_VERSION (0,10,20))
 			if (lerror->domain == GST_STREAM_ERROR) {
 				if (lerror->code == GST_STREAM_ERROR_DECRYPT ||
 				    lerror->code == GST_STREAM_ERROR_DECRYPT_NOKEY) {
-					/* also extract metadata from encrypted streams */
-
+					/* Set content as being encrypted */
 					extractor->is_content_encrypted = TRUE;
 
-					g_free (error_message);
+					g_message ("Encrypted stream found: '%s' (%s)",
+					           lerror ? lerror->message : "Unknown error",
+					           error_debug_message);
+
+					g_free (error_debug_message);
 					g_error_free (lerror);
-					break;
+
+					/* According to GStreamer guys we shouldn't keep on trying
+					 * to read tags when a _DECRYPT error is received. In this
+					 * case, we should consider all tags read up to the point
+					 * when we received the encryption error. */
+					gst_message_unref (message);
+					return TRUE;
 				}
 			}
 #endif
 
+			g_warning ("Error in GStreamer: '%s' (%s)",
+			           lerror ? lerror->message : "Unknown error",
+			           error_debug_message);
+
 			gst_message_unref (message);
-			g_warning ("Got error :%s", error_message);
-			g_free (error_message);
+			g_free (error_debug_message);
 			g_error_free (lerror);
 
 			return FALSE;
@@ -1177,9 +1199,9 @@ create_decodebin_pipeline (MetadataExtractor *extractor,
 	}
 
 	g_signal_connect (G_OBJECT (bin),
-			  "new-decoded-pad",
-			  G_CALLBACK (dbin_dpad_cb),
-			  extractor);
+	                  "new-decoded-pad",
+	                  G_CALLBACK (dbin_dpad_cb),
+	                  extractor);
 
 	gst_bin_add (GST_BIN (pipeline), filesrc);
 	gst_bin_add (GST_BIN (pipeline), bin);
@@ -1286,7 +1308,7 @@ tracker_extract_gstreamer (const gchar *uri,
 			return;
 		}
 
-		add_stream_tags(extractor);
+		add_stream_tags (extractor);
 		gst_element_set_state (extractor->pipeline, GST_STATE_READY);
 		gst_element_get_state (extractor->pipeline, NULL, NULL, 5 * GST_SECOND);
 		g_list_foreach (extractor->fsinks, unlink_fsink, extractor);
@@ -1358,7 +1380,7 @@ extract_gstreamer_svg (const gchar          *uri,
 
 static void
 extract_gstreamer_guess (const gchar          *uri,
-                        TrackerSparqlBuilder *preupdate,
+                         TrackerSparqlBuilder *preupdate,
                          TrackerSparqlBuilder *metadata)
 {
 	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_GUESS);
