@@ -66,7 +66,9 @@ static TrackerExtractData data[] = {
  * version because the TextOutputDev allows us to extract text and metadata much
  * faster than the default CairoOutputDev that poppler-glib uses in case it got
  * compiled with support for Cairo. Regretfully can't this be selected at
- * runtime in the poppler-glib bindings. Apologies to the GObject/GLib fans. */
+ * runtime in the poppler-glib bindings. Apologies to the GObject/GLib fans.
+ * Correction: Since recent versions of poppler-glib this can actually be
+ * selected. */
 
 static gchar *
 unicode_to_char (Unicode *unicode,
@@ -269,23 +271,26 @@ page_get_size (Page    *page,
                gdouble *width,
                gdouble *height)
 {
-  gdouble page_width, page_height;
-  gint rotate;
+	gdouble page_width, page_height;
+	gint rotate;
 
-  rotate = page->getRotate ();
+	rotate = page->getRotate ();
 
-  if (rotate == 90 || rotate == 270) {
-    page_height = page->getCropWidth ();
-    page_width = page->getCropHeight ();
-  } else {
-    page_width = page->getCropWidth ();
-    page_height = page->getCropHeight ();
-  }
+	if (rotate == 90 || rotate == 270) {
+		page_height = page->getCropWidth ();
+		page_width = page->getCropHeight ();
+	} else {
+		page_width = page->getCropWidth ();
+		page_height = page->getCropHeight ();
+	}
 
-  if (width != NULL)
-    *width = page_width;
-  if (height != NULL)
-    *height = page_height;
+	if (width != NULL) {
+		*width = page_width;
+	}
+
+	if (height != NULL) {
+		*height = page_height;
+	}
 }
 
 static gchar *
@@ -740,61 +745,50 @@ extract_pdf (const gchar          *uri,
 			}
 
 			if (xd->address || xd->country || xd->city) {
-				tracker_sparql_builder_predicate (metadata, "mlo:location");
+				gchar *addruri;
+
+				tracker_sparql_builder_predicate (metadata, "slo:location");
 
 				tracker_sparql_builder_object_blank_open (metadata);
 				tracker_sparql_builder_predicate (metadata, "a");
-				tracker_sparql_builder_object (metadata, "mlo:GeoPoint");
+				tracker_sparql_builder_object (metadata, "slo:GeoLocation"); /* GeoLocation */
+
+				addruri = tracker_sparql_get_uuid_urn ();
+
+				tracker_sparql_builder_predicate (metadata, "slo:postalAddress");
+				tracker_sparql_builder_object_iri (metadata, addruri);
+
+				tracker_sparql_builder_object_blank_close (metadata); /* GeoLocation */
+
+				tracker_sparql_builder_insert_open (preupdate, NULL);
+				tracker_sparql_builder_subject_iri (preupdate, addruri);
+
+				g_free (addruri);
+
+				tracker_sparql_builder_predicate (preupdate, "a");
+				tracker_sparql_builder_object (preupdate, "nco:PostalAddress");
 
 				if (xd->address) {
-					tracker_sparql_builder_predicate (metadata, "mlo:address");
-					tracker_sparql_builder_object_unvalidated (metadata, xd->address);
+					tracker_sparql_builder_predicate (preupdate, "nco:streetAddress");
+					tracker_sparql_builder_object_unvalidated (preupdate, xd->address);
 				}
 
 				if (xd->state) {
-					tracker_sparql_builder_predicate (metadata, "mlo:state");
-					tracker_sparql_builder_object_unvalidated (metadata, xd->state);
+					tracker_sparql_builder_predicate (preupdate, "nco:region");
+					tracker_sparql_builder_object_unvalidated (preupdate, xd->state);
 				}
 
 				if (xd->city) {
-					tracker_sparql_builder_predicate (metadata, "mlo:city");
-					tracker_sparql_builder_object_unvalidated (metadata, xd->city);
+					tracker_sparql_builder_predicate (preupdate, "nco:locality");
+					tracker_sparql_builder_object_unvalidated (preupdate, xd->city);
 				}
 
 				if (xd->country) {
-					tracker_sparql_builder_predicate (metadata, "mlo:country");
-					tracker_sparql_builder_object_unvalidated (metadata, xd->country);
+					tracker_sparql_builder_predicate (preupdate, "nco:country");
+					tracker_sparql_builder_object_unvalidated (preupdate, xd->country);
 				}
 
-				tracker_sparql_builder_predicate (metadata, "mlo:asPostalAddress");
-				tracker_sparql_builder_object_blank_open (metadata); /* PostalAddress */
-
-				tracker_sparql_builder_predicate (metadata, "a");
-				tracker_sparql_builder_object (metadata, "nco:PostalAddress");
-
-				if (xd->address) {
-					tracker_sparql_builder_predicate (metadata, "nco:streetAddress");
-					tracker_sparql_builder_object_unvalidated (metadata, xd->address);
-				}
-
-				if (xd->state) {
-					tracker_sparql_builder_predicate (metadata, "nco:region");
-					tracker_sparql_builder_object_unvalidated (metadata, xd->state);
-				}
-
-				if (xd->city) {
-					tracker_sparql_builder_predicate (metadata, "nco:locality");
-					tracker_sparql_builder_object_unvalidated (metadata, xd->city);
-				}
-
-				if (xd->country) {
-					tracker_sparql_builder_predicate (metadata, "nco:country");
-					tracker_sparql_builder_object_unvalidated (metadata, xd->country);
-				}
-
-				tracker_sparql_builder_object_blank_close (metadata); /* PostalAddress */
-
-				tracker_sparql_builder_object_blank_close (metadata);
+				tracker_sparql_builder_insert_close (preupdate);
 			}
 
 			/* PDF keywords aren't used ATM (why not?) */
